# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Simple Login
# Copyright (C) 2016 byteShaft
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import json

from django.conf import settings
from django.core.urlresolvers import get_callable
from django.contrib.auth import get_user_model
from rest_framework import generics, exceptions, status, response, views

from simple_login.serializers import (
    ActivationKeyRequestSerializer,
    PasswordResetRequestSerializer,
    PasswordChangeSerializer,
    StatusSerializer,
    ActivationValidationSerializer,
    LoginSerializer,
    RetrieveUpdateDestroyProfileValidationSerializer,
)
from simple_login.views.base import (
    BaseAPIView,
    ProfileBaseAPIView,
    AuthenticatedRequestBaseAPIView,
)
from simple_login import models
from simple_login.utils.otp import OTPHandler
from simple_login.utils.auth import AuthMethod
from simple_login.utils import social
from simple_login.utils.user import UserHelpers
from simple_login import serializers


SERIALIZER = get_callable(settings.AUTH_USER_SERIALIZER) if settings.AUTH_USER_SERIALIZER else \
    serializers.AuthUserSerializer


def get_unique_username(user_model, username_desired, append_id=0):
    username = username_desired
    if user_model.objects.filter(username=username).count() > 0:
        username = '{}-{}'.format(username, append_id)
        get_unique_username(user_model, username, append_id)
    return username


class TwitterLoginAPIView(views.APIView):
    def post(self, *args, **kwargs):
        serializer = serializers.TwitterLoginSerializer(data=self.request.data)
        serializer.is_valid(True)
        resp, data = social.login_twitter(serializer.data.get('access_token'),
                                          serializer.data.get('access_token_secret'))
        if resp['status'] == str(status.HTTP_200_OK):
            data_json = json.loads(data.decode())
            obj, created = models.TwitterLink.objects.get_or_create(unique_id=data_json['id_str'])
            if created:
                user = get_user_model().objects.create_user(
                    username=get_unique_username(get_user_model(),
                                                 data_json['screen_name'].lower()))
                obj.user = user
                obj.save()
            serializer = SERIALIZER(instance=obj.user)
            return response.Response(data=serializer.data, status=status.HTTP_200_OK)
        return response.Response(data=json.loads(data.decode()),
                                 status=status.HTTP_400_BAD_REQUEST)


class FacebookLoginAPIView(views.APIView):
    def post(self, *args, **kwargs):
        validator = serializers.FacebookLoginSerializer(data=self.request.data)
        validator.is_valid(True)
        resp = social.login_facebook(validator.data.get('access_token'))
        if resp.status_code == status.HTTP_200_OK:
            data_json = resp.json()
            obj, created = models.FacebookLink.objects.get_or_create(unique_id=data_json['id'])
            if created:
                user = get_user_model().objects.create_user(
                    username=get_unique_username(get_user_model(),
                                                 data_json['first_name'].lower()))
                obj.user = user
                obj.save()
                serializer = SERIALIZER(instance=user)
                return response.Response(data=serializer.data, status=status.HTTP_201_CREATED)
            serializer = SERIALIZER(instance=obj.user)
            return response.Response(data=serializer.data, status=status.HTTP_200_OK)
        return response.Response(data=resp.json(), status=status.HTTP_400_BAD_REQUEST)


class RegisterAPIView(generics.CreateAPIView):
    def post(self, request, *args, **kwargs):
        if AuthMethod.email_only() and 'email' not in self.request.data:
            raise exceptions.ValidationError({'email': ['This field is required.']})
        elif AuthMethod.username_only() and 'username' not in self.request.data:
            raise exceptions.ValidationError({'username': ['This field is required.']})
        elif 'email' not in self.request.data and 'username' not in self.request.data:
            raise exceptions.ValidationError('Must provide either `email` or `username`')
        response = super().post(request, *args, **kwargs)
        if response.status_code == status.HTTP_201_CREATED and not AuthMethod.email_only():
            response.data.update(
                {'token': UserHelpers(get_user_model(),
                                      id=int(response.data['id'])).get_auth_token()})
        return response


class ActivationAPIView(ProfileBaseAPIView):
    validation_class = ActivationValidationSerializer

    def post(self, *args, **kwargs):
        super().post(*args, **kwargs)
        self.user_account.activate()
        return response.Response(data=self.get_user_profile_data_with_token(),
                                 status=status.HTTP_200_OK)


class ActivationKeyRequestAPIView(BaseAPIView):
    validation_class = ActivationKeyRequestSerializer

    def post(self, *args, **kwargs):
        super().post(*args, **kwargs)
        otp_handler = OTPHandler(self.get_user())
        otp_handler.generate_and_send_account_activation_otps(commit=True)
        return response.Response(status=status.HTTP_200_OK)


class LoginAPIView(ProfileBaseAPIView):
    validation_class = LoginSerializer

    def post(self, *args, **kwargs):
        super().post(*args, **kwargs)
        return response.Response(data=self.get_user_profile_data_with_token(), status=status.HTTP_200_OK)


class PasswordResetRequestAPIView(BaseAPIView):
    validation_class = PasswordResetRequestSerializer

    def post(self, *args, **kwargs):
        super().post(*args, **kwargs)
        self.user_account.generate_and_send_password_reset_email_otp()
        return response.Response(status=status.HTTP_200_OK)


class PasswordChangeAPIView(BaseAPIView):
    validation_class = PasswordChangeSerializer

    def post(self, *args, **kwargs):
        super().post(*args, **kwargs)
        self.user_account.change_password(self.serializer.data.get('new_password'))
        return response.Response(status=status.HTTP_200_OK)


class StatusAPIView(BaseAPIView):
    validation_class = StatusSerializer

    def post(self, *args, **kwargs):
        super().post(*args, **kwargs)
        return response.Response(status=status.HTTP_200_OK)


class RetrieveUpdateDestroyProfileAPIView(AuthenticatedRequestBaseAPIView):
    validation_class = RetrieveUpdateDestroyProfileValidationSerializer
    http_method_names = ['put', 'patch', 'get', 'delete']

    def get(self, *args, **kwargs):
        serializer_class = self.get_serializer_class()
        serializer = serializer_class(self.get_auth_user())
        return response.Response(serializer.data, status=status.HTTP_200_OK)

    def delete(self, *args, **kwargs):
        self.get_auth_user().delete()
        return response.Response(status=status.HTTP_204_NO_CONTENT)

    def put(self, *args, **kwargs):
        super().put(*args, **kwargs)
        serializer = self.update_fields_with_request_data()
        self.ensure_password_hashed()
        return response.Response(data=serializer.data, status=status.HTTP_200_OK)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Simple Login
# Copyright (C) 2016 byteShaft
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from django.conf import settings

from simple_login.utils import send_activation_email, generate_random_key


class OTPHandler:
    def __init__(self, user_model_instance):
        self.instance = user_model_instance

    def _hasattr(self, attr):
        return self._getattr(attr) is not None

    def _getattr(self, attr):
        return getattr(self.instance, attr)

    def _setattr(self, key, value):
        setattr(self.instance, key, value)

    def generate_and_send_account_activation_otps(self, commit=False):
        if self._hasattr('account_activation_sms_otp'):
            self._setattr(
                'account_activation_sms_otp',
                self.generate_sms_otp(
                    self._getattr(settings.ACCOUNT_MOBILE_NUMBER_FIELD))
            )
        if self._hasattr('account_activation_email_otp'):
            self._setattr(
                'account_activation_email_otp',
                self.generate_email_otp()
            )
            send_activation_email(
                self.instance.email,
                self.instance.account_activation_email_otp
            )
        if commit:
            self.instance.save()

    @staticmethod
    def _get_email_otp_generator():
        try:
            return settings.ACCOUNT_ACTIVATION_EMAIL_OTP_CALLABLE
        except AttributeError:
            return generate_random_key

    @staticmethod
    def generate_email_otp():
        return OTPHandler._get_email_otp_generator()()

    @staticmethod
    def _get_sms_otp_generator():
        try:
            return settings.ACCOUNT_ACTIVATION_SMS_OTP_CALLABLE
        except AttributeError:
            return None

    @staticmethod
    def generate_sms_otp(mobile_number):
        return OTPHandler._get_sms_otp_generator()(mobile_number)



from django.conf import settings

from simple_login import AUTH_METHOD_EMAIL, AUTH_METHOD_USERNAME, AUTH_METHOD_EMAIL_USERNAME


class AuthMethod:
    @staticmethod
    def email_only():
        if not settings.ACCOUNT_AUTH_METHOD:
            return True
        return settings.ACCOUNT_AUTH_METHOD == AUTH_METHOD_EMAIL

    @staticmethod
    def username_only():
        return settings.ACCOUNT_AUTH_METHOD == AUTH_METHOD_USERNAME

    @staticmethod
    def email_or_username():
        return settings.ACCOUNT_AUTH_METHOD == AUTH_METHOD_EMAIL_USERNAME


def get_query(data):
    assert isinstance(data, dict)
    if AuthMethod.email_only():
        if 'email' not in data:
            raise ValueError('`email` is required')
        query = {'email': data['email']}
    elif AuthMethod.username_only():
        if 'username' not in data:
            raise ValueError('`username` is required')
        query = {'username': data['username']}
    elif AuthMethod.email_or_username():
        if 'username' in data and 'email' not in data:
            query = {'username': data['username']}
        elif 'username' not in data and 'email' in data:
            query = {'email': data['email']}
        elif 'username' in data and 'email' in data:
            query = {'username': data['username'], 'email': data['email']}
        else:
            raise ValueError('`username` or `email` must be provided')
    else:
        query = {}

    return query



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Simple Login
# Copyright (C) 2016 byteShaft
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from rest_framework import exceptions, status


class NotModified(exceptions.APIException):
    status_code = status.HTTP_304_NOT_MODIFIED
    default_detail = 'Nothing to change.'


class Forbidden(exceptions.APIException):
    status_code = status.HTTP_403_FORBIDDEN
    default_detail = 'Not allowed.'



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Simple Login
# Copyright (C) 2016 byteShaft
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from django.conf import settings
from django.contrib.auth import get_user_model
from rest_framework import (
    exceptions as drf_exceptions,
    serializers
)

from simple_login import KEY_DEFAULT_VALUE
from simple_login.serializers.base import BaseSerializer
from simple_login.exceptions import Forbidden
from simple_login.utils.auth import get_query

OTP_METHOD_EMAIL = 'email'
OTP_METHOD_SMS = 'sms'

try:
    OTP_METHODS = [method.lower() for method in settings.OTP_METHODS]
except AttributeError:
    OTP_METHODS = [OTP_METHOD_EMAIL]


class AuthUserSerializer(serializers.ModelSerializer):
    password = serializers.CharField(write_only=True)

    class Meta:
        model = get_user_model()
        fields = ('id', 'username', 'email', 'password')


class ActivationKeyRequestSerializer(BaseSerializer):
    def validate(self, attrs):
        super().validate(attrs)
        self.raise_if_user_does_not_exist()
        self.raise_if_user_already_activated()
        self.raise_if_user_deactivated_by_admin()
        return attrs


class ActivationValidationSerializer(BaseSerializer):
    email_otp = serializers.IntegerField(
        label='Email OTP', required=OTP_METHOD_EMAIL in OTP_METHODS)
    sms_otp = serializers.IntegerField(label='SMS OTP', required=OTP_METHOD_SMS in OTP_METHODS)

    def _raise_if_email_otp_invalid(self):
        user = self.user_model.objects.get(**get_query(self.attrs))
        otp = user.account_activation_email_otp
        if otp == KEY_DEFAULT_VALUE or otp != int(self.email_otp):
            raise serializers.ValidationError('Invalid email OTP.')

    def _raise_if_sms_otp_invalid(self):
        user = self.user_model.objects.get(**get_query(self.attrs))
        otp = user.account_activation_sms_otp
        if otp == KEY_DEFAULT_VALUE or otp != int(self.sms_otp):
            raise serializers.ValidationError('Invalid sms OTP.')

    def validate(self, attrs):
        super().validate(attrs)
        self.email_otp = attrs.get('email_otp')
        self.sms_otp = attrs.get('sms_otp')
        self.raise_if_user_does_not_exist()
        self.raise_if_user_already_activated()
        self.raise_if_user_deactivated_by_admin()
        if OTP_METHOD_EMAIL in OTP_METHODS:
            self._raise_if_email_otp_invalid()
        if OTP_METHOD_SMS in OTP_METHODS:
            self._raise_if_sms_otp_invalid()
        return attrs


class LoginSerializer(BaseSerializer):
    password = serializers.CharField(label='Password')

    def _raise_if_password_invalid(self):
        user = self.user_model.objects.get(**get_query(self.attrs))
        if not user.check_password(self.password):
            raise drf_exceptions.AuthenticationFailed('Invalid password.')

    def validate(self, attrs):
        super().validate(attrs)
        self.password = attrs.get('password')
        self.raise_if_user_does_not_exist()
        self.raise_if_user_not_activated()
        self.raise_if_user_deactivated_by_admin()
        self._raise_if_password_invalid()
        return attrs


class PasswordResetRequestSerializer(BaseSerializer):
    def validate(self, attrs):
        super().validate(attrs)
        self.raise_if_user_does_not_exist()
        self.raise_if_user_deactivated_by_admin()
        return attrs


class PasswordChangeSerializer(BaseSerializer):
    email_otp = serializers.IntegerField(label='Password reset email OTP')
    new_password = serializers.CharField(label='New password')

    def _raise_if_password_reset_email_otp_invalid(self):
        user = self.user_model.objects.get(**get_query(self.attrs))
        key = user.password_reset_email_otp
        if key == KEY_DEFAULT_VALUE or key != int(self.email_otp):
            raise serializers.ValidationError('Invalid email OTP')

    def validate(self, attrs):
        super().validate(attrs)
        self.email_otp = attrs.get('email_otp')
        self.raise_if_user_does_not_exist()
        self.raise_if_user_deactivated_by_admin()
        self._raise_if_password_reset_email_otp_invalid()
        return attrs


class StatusSerializer(BaseSerializer):
    def validate(self, attrs):
        super().validate(attrs)
        self.raise_if_user_does_not_exist()
        self.raise_if_user_not_activated()
        self.raise_if_user_deactivated_by_admin()
        return attrs


class RetrieveUpdateDestroyProfileValidationSerializer(BaseSerializer):
    email = serializers.CharField(label='Email', required=False)

    def validate(self, attrs):
        super().validate(attrs)
        if self.email:
            raise Forbidden('Not allowed to change email.')
        return attrs


class TwitterLoginSerializer(serializers.Serializer):
    access_token = serializers.CharField(required=True)
    access_token_secret = serializers.CharField(required=True)


class FacebookLoginSerializer(serializers.Serializer):
    access_token = serializers.CharField(required=True)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Simple Login
# Copyright (C) 2016 byteShaft
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import inspect

from rest_framework.authtoken.models import Token

from simple_login import KEY_DEFAULT_VALUE
from simple_login.utils import (
    generate_random_key,
    send_password_reset_email,
    send_activation_email,
)


class UserHelpers:
    def __init__(self, model_class_or_instance, **kwargs):
        if inspect.isclass(model_class_or_instance):
            self.user = model_class_or_instance.objects.get(**kwargs)
        else:
            self.user = model_class_or_instance

    def commit_changes(self):
        self.user.save()

    def _generate_and_save_password_reset_email_otp(self):
        key = generate_random_key()
        self._set_password_reset_email_otp(key)
        return key

    def _set_password_reset_email_otp(self, key):
        self.user.password_reset_email_otp = key
        self.user.save()

    def _get_email(self):
        return self.user.email

    def generate_and_send_password_reset_email_otp(self):
        key = self._generate_and_save_password_reset_email_otp()
        send_password_reset_email(self._get_email(), key)

    def hash_password(self, commit=True):
        self.user.set_password(self.user.password)
        if commit:
            self.commit_changes()

    def change_password(self, new_password, commit=True):
        self.user.set_password(new_password)
        self.user.password_reset_email_otp = KEY_DEFAULT_VALUE
        if commit:
            self.user.save()

    def generate_auth_token(self):
        Token.objects.create(user=self.user)

    def get_auth_token(self):
        # print(self.user)
        return Token.objects.get(user=self.user).key

    def activate(self, commit=True):
        self.user.is_active = True
        self.user.account_activation_email_otp = KEY_DEFAULT_VALUE
        self.user.account_activation_sms_otp = KEY_DEFAULT_VALUE
        if commit:
            self.user.save()

    def _generate_and_save_account_activation_email_otp(self):
        key = generate_random_key()
        self.user.account_activation_email_otp = key
        self.user.save()
        return key

    def generate_and_send_account_activation_email_otp(self):
        send_activation_email(
            self.user.email,
            self._generate_and_save_account_activation_email_otp()
        )

    def is_active(self):
        return self.user.is_active

    def set_active(self, boolean, commit=True):
        self.user.is_active = boolean
        if commit:
            self.commit_changes()

    def is_admin(self):
        return self.user.is_admin



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Simple Login
# Copyright (C) 2016 byteShaft
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import threading

from django.conf import settings
from django.core.mail import send_mail


def _send_mail_in_worker_thread(
        title,
        body,
        recipients,
        auth_user=settings.EMAIL_HOST_USER,
        fail_silently=False
):
    threading.Thread(
        target=send_mail,
        args=(title, body, auth_user, recipients),
        kwargs={'fail_silently': fail_silently}
    ).start()


def send_password_reset_email(email, key):
    _send_mail_in_worker_thread(
        '{}: Password reset'.format(settings.APP_NAME),
        'Password reset key: {}'.format(key),
        [str(email)]
    )


def send_activation_email(email, key):
    _send_mail_in_worker_thread(
        '{}: Account activation'.format(settings.APP_NAME),
        'Account activation key: {}'.format(key),
        [str(email)]
    )



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Simple Login
# Copyright (C) 2016 byteShaft
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from django.conf import settings
from django.contrib.auth.models import AbstractBaseUser, PermissionsMixin
from django.db import models
from django.db.models.signals import post_save

from simple_login import KEY_DEFAULT_VALUE
from simple_login.managers import SimpleUserManager
from simple_login.models.utils import process_save


class TwitterLink(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE, null=True)
    unique_id = models.CharField(max_length=255, blank=False)


class FacebookLink(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.CASCADE, null=True)
    unique_id = models.CharField(max_length=255, blank=False)


class BaseUser(AbstractBaseUser, PermissionsMixin):
    username = models.CharField(max_length=255, blank=True, unique=True, null=True)
    email = models.EmailField(max_length=255, blank=True, unique=True, null=True)

    is_admin = models.BooleanField(default=False)
    is_active = models.BooleanField(default=True)
    date_joined = models.DateTimeField(auto_now_add=True, blank=False)

    # OTP records.
    account_activation_email_otp = models.IntegerField(default=KEY_DEFAULT_VALUE)
    account_activation_sms_otp = models.IntegerField(default=KEY_DEFAULT_VALUE)
    password_reset_email_otp = models.IntegerField(default=KEY_DEFAULT_VALUE)
    password_reset_sms_otp = models.IntegerField(default=KEY_DEFAULT_VALUE)

    objects = SimpleUserManager()

    USERNAME_FIELD = 'username'

    def get_full_name(self):
        return self.username if self.username else self.email

    def get_short_name(self):
        return self.username if self.username else self.email

    def __str__(self):
        return self.username if self.username else self.email

    def has_perm(self, perm, obj=None):
        return True

    def has_module_perms(self, app_label):
        return True

    @property
    def is_staff(self):
        return self.is_admin

    class Meta:
        abstract = True


post_save.connect(process_save, sender=settings.AUTH_USER_MODEL)



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Simple Login
# Copyright (C) 2016 byteShaft
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from simple_login.views.api import (
    ActivationAPIView,
    ActivationKeyRequestAPIView,
    LoginAPIView,
    PasswordResetRequestAPIView,
    PasswordChangeAPIView,
    StatusAPIView,
    RetrieveUpdateDestroyProfileAPIView,
    RegisterAPIView,
    TwitterLoginAPIView,
    FacebookLoginAPIView,
)

__all__ = [
    'ActivationAPIView',
    'ActivationKeyRequestAPIView',
    'LoginAPIView',
    'PasswordResetRequestAPIView',
    'PasswordChangeAPIView',
    'StatusAPIView',
    'RetrieveUpdateDestroyProfileAPIView',
    'RegisterAPIView',
    'TwitterLoginAPIView',
    'FacebookLoginAPIView'
]



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Simple Login
# Copyright (C) 2016 byteShaft
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from rest_framework import exceptions, serializers

from simple_login.exceptions import NotModified, Forbidden
from simple_login.utils.auth import AuthMethod, get_query
from simple_login.models import BaseUser
from simple_login import KEY_DEFAULT_VALUE


class BaseSerializer(serializers.Serializer):

    email = serializers.EmailField(label='Email', required=AuthMethod.email_only())
    username = serializers.CharField(label='Username', required=AuthMethod.username_only())

    def __init__(self, user_model, **kwargs):
        super().__init__(**kwargs)
        self.email = None
        self.attrs = None
        if not issubclass(user_model, BaseUser):
            raise serializers.ValidationError(
                'user_model must be a subclass of simple_login.models.BaseUser'
            )
        self.user_model = user_model

    def create(self, validated_data):
        pass

    def update(self, instance, validated_data):
        pass

    def validate(self, attrs):
        """A common call that's part of every request."""
        self.attrs = attrs
        self.email = attrs.get('email')
        self.username = attrs.get('username')

    def raise_if_user_does_not_exist(self):
        try:
            return self.user_model.objects.get(**get_query(self.attrs))
        except self.user_model.DoesNotExist:
            raise exceptions.NotFound('User with email \'{}\' does not exist.'.format(self.email))

    def raise_if_user_already_activated(self):
        user = self.user_model.objects.get(**get_query(self.attrs))
        if user.is_active:
            raise NotModified('User already activated.')

    def raise_if_user_not_activated(self):
        user = self.user_model.objects.get(**get_query(self.attrs))
        if not user.is_active and user.account_activation_email_otp != KEY_DEFAULT_VALUE:
            raise Forbidden('User not active.')

    def raise_if_user_deactivated_by_admin(self):
        user = self.user_model.objects.get(**get_query(self.attrs))
        if not user.is_active and user.account_activation_email_otp == KEY_DEFAULT_VALUE:
            raise Forbidden('User deactivated by admin.')



# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-

#
# Simple Login
# Copyright (C) 2016 byteShaft
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

from rest_framework import permissions
from rest_framework.views import APIView

from simple_login.utils import UserHelpers
from simple_login.utils.auth import get_query


class BaseAPIView(APIView):
    user_model = None
    serializer = None
    serializer_class = None
    validation_class = None

    def post(self, *args, **kwargs):
        """Validate parameters for a request.

        Ensure to call super().post(*args, **kwargs) when this method is
        overridden.
        """
        self.validate_request_parameters()

    def put(self, *args, **kwargs):
        """Validate parameters for a request.

        Ensure to call super().put(*args, **kwargs) when this method is
        overridden.
        """
        self.validate_request_parameters()

    @property
    def user_account(self):
        """Return account helpers object by reading email from serializer
        data."""
        return UserHelpers(self.user_model, **get_query(self.serializer.data))

    def get_user(self):
        if not self.request.user.is_anonymous():
            return self.get_auth_user()
        return self.user_account.user

    def get_auth_user(self):
        return self.request.user

    def get_serializer_class(self):
        """Return the data serializer class.

        One may override this method to return custom serializer class.
        """
        return self.serializer_class

    def get_validation_class(self):
        return self.validation_class

    def get_user_model(self):
        """Return the User account model.

        One may override this method to return a custom user account model.
        """
        return self.user_model

    def validate_request_parameters(self, raise_exception=True):
        if self.validation_class:
            serializer_class = self.get_validation_class()
        else:
            serializer_class = self.get_serializer_class()
        self.serializer = serializer_class(user_model=self.user_model, data=self.request.data)
        self.serializer.is_valid(raise_exception=raise_exception)


class ProfileBaseAPIView(BaseAPIView):
    def get_user_profile_data_with_token(self):
        serializer_class = self.get_serializer_class()
        serializer = serializer_class(instance=self.user_account.user)
        data = serializer.data
        data.update({'token': self.user_account.get_auth_token()})
        return data


class AuthenticatedRequestBaseAPIView(BaseAPIView):
    permission_classes = (permissions.IsAuthenticated, )

    def ensure_password_hashed(self):
        password = self.request.data.get('password')
        if password:
            self.request.user.set_password(password)
            self.request.user.save()

    def update_fields_with_request_data(self):
        serializer_class = self.get_serializer_class()
        serializer = serializer_class(
            instance=self.request.user,
            data=self.request.data,
            partial=True
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return serializer



import json

from django.conf import settings
import oauth2 as oauth
import requests

FB_BASE_URL = 'https://graph.facebook.com/v2.5/me'


def login_twitter(access_key, access_secret):
    consumer_key = settings.TWITTER_CONSUMER_KEY
    consumer_secret = settings.TWITTER_CONSUMER_SECRET

    consumer = oauth.Consumer(key=consumer_key, secret=consumer_secret)
    access_token = oauth.Token(key=access_key, secret=access_secret)
    client = oauth.Client(consumer, access_token)

    timeline_endpoint = "https://api.twitter.com/1.1/account/verify_credentials.json"
    return client.request(timeline_endpoint)

    # return json.loads(data.decode())


def login_facebook(access_token):
    url = '{}?fields=id,first_name,last_name,email&access_token={}'.format(
        FB_BASE_URL, access_token)
    return requests.get(url)



import os
from setuptools import setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-simple-login',
    version='0.7.0',
    packages=['simple_login', 'simple_login.apps',
              'simple_login.utils', 'simple_login.views',
              'simple_login.models', 'simple_login.managers',
              'simple_login.exceptions', 'simple_login.serializers'],
    url='https://github.com/byteshaft/django-simple-login',
    license='GNU GPL Version 3',
    author='Omer Akram',
    author_email='om26er@gmail.com',
    description='A simple django app to make accounts creation a breeze.',
    long_description=README,
    download_url='https://github.com/byteShaft/django-simple-login/tarball/'
                 '0.7.0',
    keywords=['django', 'accounts'],
    classifiers=[],
    install_requires=['django', 'djangorestframework', 'oauth2', 'requests'],
)
